import { merge } from 'lodash'
import configs from '@/config'
import { Axios } from './axios'
import { ContentTypeEnum, RequestMethodsEnum } from '@/enums/requestEnums'
import type { AxiosHooks } from './type'
import { clearAuthInfo, getToken } from '../auth'
import feedback from '../feedback'
import NProgress from 'nprogress'
import { AxiosError, type AxiosRequestConfig } from 'axios'
import router from '@/router'

function guolv(data:any){
    let obj:any = {}
    for(let key in data){
        if(data[key] !=='' && data[key] !==null){
            obj[key] = data[key]
        }
    }
    return obj
}

// 处理axios的钩子函数
const axiosHooks: AxiosHooks = {
    requestInterceptorsHook(config: any) {
        NProgress.start()
        const { withToken, isParamsToData } = config.requestOptions;
        if (!config.params) config.params = {};
        const params = guolv(config.params) || {};
        const headers = guolv(config.headers) || {}

        // POST请求下如果无data，则将params视为data
        if (
            isParamsToData &&
            !Reflect.has(config, 'data') &&
            config.method?.toUpperCase() === RequestMethodsEnum.POST
        ) {
            config.data = params
            config.params = {};
        }

        // 添加token
        if (withToken) {
            const token = getToken();
            config.data = {
                ...config.data
            }
            headers['Authorization'] = token;
        }
        

        if (config.url.indexOf("/foxpsdData/") != -1) {
            config.params.method = config.url.replace("/foxpsdData", "")
        }

        if (headers['Content-Type'] == 'application/x-www-form-urlencoded') {
            const _params: any = new URLSearchParams();
            for (let key in config.data) {
                if (config.data[key] != null && config.data[key] != 'null') {
                    _params.append(key, config.data[key])
                }
            }
            config.data = _params;
        }


        config.headers = headers
        return config
    },

    requestInterceptorsCatchHook(err) {
        console.log(err);
        NProgress.done()
        return err
    },

    async responseInterceptorsHook(response: any) {
        NProgress.done()
        const { isTransformResponse, isReturnDefaultResponse } = response.config.requestOptions

        //返回默认响应，当需要获取响应头及其他数据时可使用
        if (isReturnDefaultResponse) {
            return response
        }
        // 是否需要对数据进行处理
        if (!isTransformResponse) {
            return response.data
        }

        const { msg, code } = response.data

        if (code == 10001) {
            feedback.msgError(msg);
            if (msg == 'token已失效') {
                router.push('/login');
            }
            return Promise.reject(response.data)
        }

        return response.data;
    },

    responseInterceptorsCatchHook(error) {
        NProgress.done()
        if (error.code !== AxiosError.ERR_CANCELED) {
            error.message && feedback.msgError(error.message)
        }
        return Promise.reject(error)
    }
}

const defaultOptions: AxiosRequestConfig = {
    //接口超时时间
    timeout: configs.timeout,
    // 基础接口地址
    baseURL: configs.baseUrl,
    //请求头
    headers: { 'Content-Type': ContentTypeEnum.FORM_URL },
    // 处理 axios的钩子函数
    axiosHooks: axiosHooks,
    // 每个接口可以单独配置
    requestOptions: {
        // 是否将params视为data参数，仅限post请求
        isParamsToData: true,
        //是否返回默认的响应
        isReturnDefaultResponse: false,
        // 需要对返回数据进行处理
        isTransformResponse: true,
        // 接口拼接地址
        urlPrefix: configs.urlPrefix,
        // 忽略重复请求
        ignoreCancelToken: true,
        // 是否携带token
        withToken: true,
        // 开启请求超时重新发起请求请求机制
        isOpenRetry: true,
        // 重新请求次数
        retryCount: 2
    }
}
function createAxios(opt?: Partial<AxiosRequestConfig>) {
    return new Axios(
        // 深度合并
        merge(defaultOptions, opt || {})
    )
}
const request = createAxios()
export default request
